; 1wire SLAVE DS18B20 							
; 10xLED										
;												
; ATmega8										
; 4 MHz											
;												
; Bernhard.Erfurt@gmx.de						
; 02/2017										
;												

; LED grn = SLAVE wird direkt angesprochen		
; LED gelb = Messung							
; LED rot  = unbekantes Function-Command		

; ATmega8 										
.include "m8def.inc"
; KONSTANTEN									
.equ TAKT						=4000000			
; 1-WIRE-ROM-ID									
.equ ROM_ID_BYTE0		= 0x28	;FAMILY				
.equ ROM_ID_BYTE1		= 0x20
.equ ROM_ID_BYTE2		= 0x17
.equ ROM_ID_BYTE3		= 0x00
.equ ROM_ID_BYTE4		= 0x00
.equ ROM_ID_BYTE5		= 0x00
.equ ROM_ID_BYTE6		= 0x00
.equ ROM_ID_BYTE7		= 0x00	; D9 CRC-DUMMY (wird automatisch korrigiert)	
; SCRATCHPAD (Beispiel							
.equ SCRATCHPAD_BYTE0	= 0xB6
.equ SCRATCHPAD_BYTE1	= 0x01
.equ SCRATCHPAD_BYTE2	= 0x4B
.equ SCRATCHPAD_BYTE3	= 0x46
.equ SCRATCHPAD_BYTE4	= 0x7F
.equ SCRATCHPAD_BYTE5	= 0xFF
.equ SCRATCHPAD_BYTE6	= 0x0A
.equ SCRATCHPAD_BYTE7	= 0x10
.equ SCRATCHPAD_BYTE8	= 0x00	; (A2) CRC-DUMMY (wird automatisch korrigiert)	

; 1-WIRE-ROM-COMMANDS							
.equ OW_COMMAND_SKIP_ROM		=0xCC
.equ OW_COMMAND_MATCH_ROM		=0x55
.equ OW_COMMAND_ROM_READ		=0x33
.equ OW_COMMAND_SEARCH_ROM		=0xF0
; 1-WIRE-FUNCTION-COMMANDS						
.equ OW_FUNCTION_CONVERT_T		=0x44		;CONVERT "T" DS19s20	
.equ OW_FUNCTION_TX_PRIVATE_DATA=0xF6
.equ OW_FUNCTION_READ_SCRATCHPAD=0xBE		; read  DS18s20	
.equ OW_FUNCTION_RECALL_EE		=0xB8
; 1-WIRE-DATENBUS								
.equ OW_PIN_NR				= 2
.equ OW_DDR					= DDRD
.equ OW_PORT				= PORTD
.equ OW_PIN					= PIND
; LED-GRUEN										
.equ LED_GRUEN_PIN_NR		= 5
.equ LED_GRUEN_DDR			= DDRD
.equ LED_GRUEN_PORT			= PORTD
; LED-GELB										
.equ LED_GELB_PIN_NR		= 6
.equ LED_GELB_DDR			= DDRD
.equ LED_GELB_PORT			= PORTD
; LED-ROT										
.equ LED_ROT_PIN_NR			= 7
.equ LED_ROT_DDR			= DDRD
.equ LED_ROT_PORT			= PORTD
; LED 0											
.equ LED_0_PIN_NR			= 0
.equ LED_0_DDR				= DDRD
.equ LED_0_PORT				= PORTD
; LED 1											
.equ LED_1_PIN_NR			= 1
.equ LED_1_DDR				= DDRD
.equ LED_1_PORT				= PORTD
; LED 2											
.equ LED_2_PIN_NR			= 3
.equ LED_2_DDR				= DDRD
.equ LED_2_PORT				= PORTD
; LED 3											
.equ LED_3_PIN_NR			= 4
.equ LED_3_DDR				= DDRD
.equ LED_3_PORT				= PORTD
; LED 4											
.equ LED_4_PIN_NR			= 0
.equ LED_4_DDR				= DDRB
.equ LED_4_PORT				= PORTB
; LED 5											
.equ LED_5_PIN_NR			= 1
.equ LED_5_DDR				= DDRC
.equ LED_5_PORT				= PORTC
; LED 6											
.equ LED_6_PIN_NR			= 2
.equ LED_6_DDR				= DDRC
.equ LED_6_PORT				= PORTC
; LED 7											
.equ LED_7_PIN_NR			= 3
.equ LED_7_DDR				= DDRC
.equ LED_7_PORT				= PORTC
; LED 8											
.equ LED_8_PIN_NR			= 4
.equ LED_8_DDR				= DDRC
.equ LED_8_PORT				= PORTC
; LED 9											
.equ LED_9_PIN_NR			= 5
.equ LED_9_DDR				= DDRC
.equ LED_9_PORT				= PORTC
; REGISTER										
.def OW_RX_BIT_ZAEHLER	= R2
.def OW_RX_BYTE			= R3
.def OW_ZAEHLER 		= R4
.def OW_STATUS			= R5

.def NULL 				= R13
.def EINS 				= R14
.def VOLL				= R15

.def temp 				= R16
.def temp1 				= R17
.def temp2 				= R18
.def temp3 				= R19
.def temp4 				= R20
.def temp5 				= R21
.def temp6 				= R22
.def temp7 				= R23
.def temp8 				= R24
.def temp9 				= R25
; SRAM-BELEGUNG ATmega8							
.equ adr_DATEN							=SRAM_START			; 0x0060 START Adresse 		

.equ adr_FREIGABE_MESSEN				=SRAM_START+10

.equ adr_OW_ZAEHLER_RP_L				=SRAM_START+100		; fr Wartungszwecke (ZAEHLERWERTE)	
.equ adr_OW_ZAEHLER_RP_H				=SRAM_START+101		; fr Wartungszwecke (ZAEHLERWERTE)	
.equ adr_OW_SEARCH_COUNTER				=SRAM_START+102
.equ adr_OW_ROM_COMAND					=SRAM_START+103
.equ adr_OW_FUNC_COMAND					=SRAM_START+104

.equ adr_OW_ROM_ID_BEREICH8				=SRAM_START+110		; (8 BYTE) ROM-ID SLAVE		
.equ adr_OW_SCRATCHPAD_BEREICH9			=SRAM_START+120		; (9 BYTE) SCRATCHPAD  		

.equ adr_OW_RX_DATA256					=SRAM_START+200		; 1wire DATA

.equ adr_OW_TX_DATA256					=SRAM_START+500		; 1wire DATA

.equ adr_SRAM_CHECK						=SRAM_START+943		; SRAM CHECK muss immer "S" sein

.cseg					;Beginn eines Code-Segmentes
.org 0					;Startadresse=0				

rjmp RESET      		; Reset Handler                           				
rjmp EXT_INT0   		; IRQ0 Handler  										
rjmp INTERRUPT_ERROR	;rjmp EXT_INT1   ; IRQ1 Handler                      	
rjmp INTERRUPT_ERROR	;rjmp TIM2_COMP  ; Timer2 Compare Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM2_OVF   ; Timer2 Overflow Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_CAPT  ; Timer1 Capture Handler            	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPA ; Timer1 CompareA Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_COMPB ; Timer1 CompareB Handler           	
rjmp INTERRUPT_ERROR	;rjmp TIM1_OVF   ; Timer1 Overflow Handler           	
rjmp INTERRUPT_ERROR	;TIM0_OVF   		; Timer0 Overflow Handler           
rjmp INTERRUPT_ERROR	;rjmp SPI_STC    ; SPI Transfer Complete Handler     	
rjmp INTERRUPT_ERROR	; USART RX Complete Handler         					
rjmp INTERRUPT_ERROR	;rjmp USART_UDRE ; UDR Empty Handler                 	
rjmp INTERRUPT_ERROR	;rjmp USART_TXC  ; USART TX Complete Handler         	
rjmp INTERRUPT_ERROR	;rjmp ADC        ; ADC Conversion Complete Handler   	
rjmp INTERRUPT_ERROR	;rjmp EE_RDY     ; EEPROM Ready Handler              	
rjmp INTERRUPT_ERROR	;rjmp ANA_COMP   ; Analog Comparator Handler         	
rjmp INTERRUPT_ERROR	;rjmp TWSI      ; Two-wire Serial Interface Handler 	
rjmp INTERRUPT_ERROR	;rjmp SPM_RDY    ; Store Program Memory Ready Handler	


; Bei unkorrekten Interrupt aufrufen erfolgt ein RESET ber WDR	
INTERRUPT_ERROR:
	rjmp INTERRUPT_ERROR
RESET:                        
; STACK initialisieren								
	ldi temp, LOW(RAMEND)
    out SPL, temp
    ldi temp, HIGH(RAMEND)
    out SPH, temp
; SRAM_CLEAR										
	rcall SRAM_CLEAR
; PORT_CLEAR										
	rcall PORT_CLEAR
; REGISTER_CLEAR									
	rcall REGISTER_CLEAR
; LED_INITIALISIERUNG								
	rcall LED_INITIALISIERUNG
; WDR_INITIALISIEREN								
	rcall WDR_INITIALISIEREN
; NEUSTART_MIT_WDR									
	rcall NEUSTART_MIT_WDR
; LED_TEST											
	rcall LED_TEST
; STARTWERTE										
	; NULL				
	clr NULL
	; EINS				
	ldi temp,1
	mov EINS,temp
	; VOLL				
	ldi temp,255
	mov VOLL,temp
	; SRAM CHECK		
	ldi temp,'S'
	STS(adr_SRAM_CHECK),temp
; 1-WIRE											
	rcall OW_INITIALISIERUNG
	rcall EXTERNER_INTERUPT_INITIALISIEREN
	rcall EXTERNER_INTERRUPT_AKTIVIEREN
	rcall PROGRAMM_CHECK
; ------------------------------------------------------------------------------
	out GIFR,VOLL	; eventuelle Pegelwechsel ignorieren	
	sei 			; interrupts freigeben
; ##############################################################################
; ##############################################################################
; ##############################################################################
LOOP:
	WDR
	rcall PROGRAMM_CHECK
	LDS temp,(adr_FREIGABE_MESSEN)
	tst temp
	breq LOOP
; ------------------------------------------------------------------------------
	STS(adr_FREIGABE_MESSEN),NULL
	cbi (LED_GELB_PORT) ,(LED_GELB_PIN_NR)		; GELB an	
	rcall WAIT_100ms
	sbi (LED_GELB_PORT) ,(LED_GELB_PIN_NR)		; GELB aus	

;	LDS temp1,(adr_OW_ZAEHLER_RP_L)			
;	LDS temp2,(adr_OW_ZAEHLER_RP_H)			
;	STS(adr_OW_SCRATCHPAD_BEREICH9+0),temp1	
;	STS(adr_OW_SCRATCHPAD_BEREICH9+1),temp2	

	LDS ZL,(adr_OW_SCRATCHPAD_BEREICH9+0)
	LDS ZH,(adr_OW_SCRATCHPAD_BEREICH9+1)
	adiw ZL,10
	STS(adr_OW_SCRATCHPAD_BEREICH9+0),ZL
	STS(adr_OW_SCRATCHPAD_BEREICH9+1),ZH

	rcall SCRATCHPAD_CRC8		; wichtig !
rjmp LOOP
; ##############################################################################
; ##############################################################################
; ##############################################################################
.include "SLAVE_SONSTIGES.asm"
.include "SLAVE_WAIT_04MHZ.asm"
.include "SLAVE_EXT_INTERRUPT.asm"
.include "SLAVE_OW.asm"

